package ru.ulstu.is.sbapp.socialNetwork.services;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import ru.ulstu.is.sbapp.socialNetwork.models.Community;
import ru.ulstu.is.sbapp.socialNetwork.models.Music;
import ru.ulstu.is.sbapp.socialNetwork.models.UserModel;
import ru.ulstu.is.sbapp.socialNetwork.repository.MyUserRepository;

import javax.persistence.EntityNotFoundException;
import java.util.List;
import java.util.Optional;

@Service
public class MyUserService {

    private MyUserRepository repo;

    private CommunityService communityService;
    private MusicService musicService;

    public MyUserService(MyUserRepository repo, MusicService musicService, CommunityService communityService) {
        this.repo = repo;
        this.musicService = musicService;
        this.communityService = communityService;
    }

    @Transactional
    public UserModel addUser(String name, String city) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("User name is null or empty");
        }
        UserModel user = new UserModel(name, city);
        return repo.save(user);
    }
    @Transactional
    public UserModel addUser(String name, String city, List<String> musics, List<String> communities) {

        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Name is null or empty");
        }

        UserModel user = new UserModel(name, city);
        UserModel result = repo.save(user);

        updateMusics(result.getId(), musics);
        return updateCommunities(user.getId(), communities);
    }

    @Transactional
    public UserModel addMusic(Long userId, Long musicId) {
        final Optional<UserModel> userOpt= repo.findById(userId);

        if (userOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("User with id [%s] is not found", musicId));
        }

        UserModel user = userOpt.get();

        final Music music = musicService.findMusic(musicId);
        if (music == null) {
            throw new EntityNotFoundException(String.format("Music with id [%s] is not found", musicId));
        }

        user.addMusic(music);
        return repo.save(user);
    }
    @Transactional
    public UserModel addCommunity(Long userId, Long communityId) {
        final Optional<UserModel> userOpt = repo.findById(userId);

        if (userOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("User with id [%s] is not found", userId));
        }

        UserModel user = userOpt.get();

        final Community community = communityService.findCommunity(communityId);
        if (community == null) {
            throw new EntityNotFoundException(String.format("Community with id [%s] is not found", communityId));
        }

        user.addCommunity(community);
        return repo.save(user);
    }

    @Transactional(readOnly = true)
    public UserModel findUser(Long id) {

        final Optional<UserModel> user = repo.findById(id);
        if (user.isEmpty()) {
            throw new EntityNotFoundException(String.format("UserModel with id [%s] is not found", id));
        }
        return user.get();
    }

//    @Transactional(readOnly = true)
//    public List<UserModel> findUserByCity(String city){
//        Query query = repo.createQuery("select u from UserModel u where u.city = :city",
//                UserModel.class).setParameter("city", city);
//        List<UserModel>  result = query.getResultList();
//       return result;
//    }
    @Transactional(readOnly = true)
    public List<UserModel> findUserByCity(String city){
        List<UserModel>  result = repo.findByCity(city);
       return result;
    }

    @Transactional(readOnly = true)
    public List<UserModel> findAllUsers() {
        return repo.findAll();
    }

    @Transactional
    public UserModel updateUser(Long id, String name) {

        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("User name is null or empty");
        }
        final Optional<UserModel> currentUserOpt = repo.findById(id);

        if(currentUserOpt.isEmpty()) {
            return null;
        }

        final UserModel currentUser = currentUserOpt.get();

        currentUser.setName(name);
        return repo.save(currentUser);
    }
    //update community, musics
    @Transactional
    public UserModel updateUser(Long id, String name,String city, List<String> musics, List<String> Communities) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("User name is null or empty");
        }
        final Optional<UserModel> currentUserOpt = repo.findById(id);

        if(currentUserOpt.isEmpty()) {
            return null;
        }

        final UserModel currentUser = currentUserOpt.get();

        currentUser.setName(name);
        currentUser.setCity(city);
        repo.save(currentUser);

        updateMusics(id, musics);
        return updateCommunities(id, Communities);
    }
    
    @Transactional
    public UserModel updateCommunities(Long userId , List<String> communities) {
        final Optional<UserModel> userOpt = repo.findById(userId);

        if (userOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("UserModel with id [%s] is not found", userId));
        }

        UserModel user = userOpt.get();

        user.deleteCommunities();

        for(String c : communities) {
            Community community = communityService.findByName(c);
            user.addCommunity(community);
        }
        return repo.save(user);
    }
    @Transactional
    public UserModel updateMusics(Long userId , List<String> musics) {
        final Optional<UserModel> userOpt = repo.findById(userId);

        if (userOpt.isEmpty()) {
            throw new EntityNotFoundException(String.format("Music with id [%s] is not found", userId));
        }

        UserModel user = userOpt.get();

        user.deleteMusics();

        for(String m : musics) {
            Music music = musicService.findByName(m);
            user.addMusic(music);
        }
        return repo.save(user);
    }


    @Transactional
    public UserModel deleteUser(Long id) {

        final Optional<UserModel> currentUser = repo.findById(id);
        if(currentUser.isEmpty()) {
            return null;
        }

        repo.deleteById(id);
        return currentUser.get();
    }

    @Transactional
    public void deleteAllUsers() {
        repo.deleteAll();
    }


}
